/*
** ===================================================================
** Nzev souboru   : flash_eeprom.c
** Mikrokontrolr  : ATmega8A,...
** Popis           : Funkce pro FLASH EEPROM pam AT45DB081D.
**
** Autor           : Luk Motyka (motylu@seznam.cz)
** Posledn zmna  : 24.04.12 22:59
** ===================================================================
*/


/*
** Hlavikov soubory
*/

#include <avr\io.h>			// hlavikov soubor pro mikrokontrolr
#include "flash_eeprom.h"   // hlavikov soubor funkc FLASH EEPROM pamti



/*
** ===================================================================
**     Nzev funkce  : flash_init
**
**     Popis         : Inicializace registr a perifri mikrokontrolru.
**
**     Parametry     : dn.
**
**     Vst. hodnota : dn.
** ===================================================================
*/

void flash_init (void)
{
	DDRB |= 0b00101110;
	PORTB |= _BV(PB2);

	TCCR1A |= 0b10000001;
	TCCR1B |= 0b00001001;	// / 1: 8-bitov Fast PWM md, bez peddliky 

	SPCR |= 0b01010010;		// SPI: povolen SPI, Master md, peddlika fclk/64
}



/*
** ===================================================================
**     Nzev funkce  : spi_rw
**
**     Popis         : Zpis a ten z/do datovho registru SPDR pro rozhran SPI.
**
**     Parametry     : 8-bitov hodnota (uint8_t) pro odesln.
**
**     Vst. hodnota : Pijat 8-bitov hodnota (uint8_t).
** ===================================================================
*/

uint8_t spi_rw (uint8_t to_send)
{
	uint8_t receive;

    SPDR = to_send;		// odesln dat

    while (!(SPSR & (_BV(SPIF))));		// ekn na nastaven pznaku SPIF

	receive = SPDR;		// pijet dat

	return receive;
}



/*
** ===================================================================
**     Nzev funkce  : flash_write
**
**     Popis         : Pm zpis dat do extern pamti (pomoc SRAM bufferu 1)
**					   po rozhran SPI.
**
**     Parametry     : Adresa strnky (page_adr = 0-4095) a bytu (byte_adr = 0-263)
**					   pamti pro zpis, poet hodnot (no_data = 1-264) a ukazatel
**					   na data pro uloen.
**
**     Vst. hodnota : dn.
** ===================================================================
*/

void flash_write (uint16_t page_adr, uint16_t byte_adr, uint16_t no_data, uint8_t *data)
{
	uint16_t i;

	if (page_adr>4095)
		page_adr = 4095;

	if (byte_adr>263)
		byte_adr = 263;
		
	if (no_data>264)
		no_data = 264;		// oeten maximln monch hodnot

	SS_LOW;

    spi_rw(0x82);	// 0x82 = opcode pro pm zpis dat pomoc bufferu 1

	spi_rw((uint8_t)(page_adr>>7));
	spi_rw(((uint8_t)(page_adr<<1))|((uint8_t)(byte_adr>>8)));
	spi_rw((uint8_t)(byte_adr));		// zasln adresy strnky a bytu

	for (i=0;i<no_data;i++)
		spi_rw(data[i]);

	SS_HIGH;
}



/*
** ===================================================================
**     Nzev funkce  : flash_read
**
**     Popis         : Pm kontinuln ten dat z extern pamti (vynechnm SRAM
**					   buffer) po rozhran SPI do registru OCR1A / 1.
**
**     Parametry     : Adresa strnky (page_adr = 0-4095) a bytu (byte_adr = 0-263)
**					   pamti pro ten a chtn poet hodnot.
**
**     Vst. hodnota : dn.
** ===================================================================
*/

void flash_read (uint16_t page_adr, uint16_t byte_adr, uint16_t no_data)
{
	uint16_t i;

	if (page_adr>4095)
		page_adr = 4095;

	if (byte_adr>263)
		byte_adr = 263;		// oeten maximln monch hodnot adres

	SS_LOW;

    spi_rw(0x03);	// 0x03 = opcode pro pm kontinuln ten dat

	spi_rw((uint8_t)(page_adr>>7));
	spi_rw(((uint8_t)(page_adr<<1))|((uint8_t)(byte_adr>>8)));
	spi_rw((uint8_t)(byte_adr));		// zasln adresy strnky a bytu

	for (i=0;i<no_data;i++)
		OCR1A = spi_rw(0x00);

	SS_HIGH;
}
